@@ -3,9 +3,9 @@ from graphviz import Digraph
 
 class CodeFlowchartGenerator:
     """
-    주어진 Python 소스 코드를 파싱(AST)하여
-    순서도를(Flowchart) 자동 생성해주는 클래스.
-    Graphviz를 이용해 PNG 이미지, DOT 파일을 생성합니다.
+    A class that automatically generates flowcharts
+    by parsing the given Python source code (AST).
+    Creates PNG images and DOT files using Graphviz.
     """
 
     def __init__(self, code: str):
@@ -13,95 +13,94 @@ class CodeFlowchartGenerator:
         self.tree = ast.parse(code)
         self.graph = Digraph(format='png')
         self.graph.attr(rankdir="TB")  # Top-to-Bottom layout
-        self.graph.attr(nodesep="0.5", ranksep="0.75")  # 노드 및 랭크 간 간격
-        self.graph.attr(size="10,10", dpi="300")  # 출력 이미지 사이즈 & 해상도
-        self.counter = 0  # 노드 ID 카운터
+        self.graph.attr(nodesep="0.5", ranksep="0.75")  # Spacing between nodes and ranks
+        self.graph.attr(size="10,10", dpi="300")  # Output image size & resolution
+        self.counter = 0  # Node ID counter
         self.function_definitions = {}
         self.function_flow_nodes = {}
-        self.node_prefix_map = {}  # 각 노드 ID에 대한 prefix 관리
+        self.node_prefix_map = {}  # Manage prefix for each node ID
 
-        # 메인 흐름을 담을 서브그래프
+        # Subgraph to contain the main flow
         self.main_cluster = Digraph(name="cluster_main")
         self.main_cluster.attr(label="Main Code", fontsize="16", style="dashed", color="blue")
 
-        # 함수 정의를 담을 서브그래프
+        # Subgraph to contain function definitions
         self.function_cluster = Digraph(name="cluster_functions")
         self.function_cluster.attr(label="Functions", fontsize="16", style="dashed", color="green")
 
-        # 함수 정의를 미리 수집
+        # Pre-collect function definitions
         self._gather_function_definitions(self.tree)
 
     def _get_unique_id(self, prefix="node"):
-        """prefix를 포함한 유니크한 노드 ID를 생성."""
+        """Generates a unique node ID with the given prefix."""
         self.counter += 1
         node_id = f"{prefix}_{self.counter}"
         self.node_prefix_map[node_id] = prefix
         return node_id
 
     def _add_node(self, label, shape="box", cluster=None, prefix="node", rank="same"):
-        """그래프(혹은 서브그래프)에 노드를 추가하고 노드의 ID를 반환."""
+        """Adds a node to the graph (or subgraph) and returns the node ID."""
         node_id = self._get_unique_id(prefix)
         if cluster:
             cluster.node(node_id, label, shape=shape, fontsize="12")
         else:
             self.graph.node(node_id, label, shape=shape, fontsize="12")
 
-        # rank 설정
+        # Set rank
         if rank:
             self.graph.body.append(f'{{rank={rank}; "{node_id}";}}')
 
         return node_id
 
     def _add_edge(self, from_node, to_node, label="", cluster=None, color="black", style="solid"):
-        """그래프(혹은 서브그래프)에 엣지를 추가."""
+        """Adds an edge to the graph (or subgraph)."""
         if cluster:
             cluster.edge(from_node, to_node, label=label, fontsize="10", color=color, style=style)
         else:
             self.graph.edge(from_node, to_node, label=label, fontsize="10", color=color, style=style)
 
     def _get_node_prefix(self, node_id):
-        """노드 ID에 대응하는 prefix를 반환."""
+        """Returns the prefix corresponding to the node ID."""
         return self.node_prefix_map.get(node_id, "unknown")
 
     def _gather_function_definitions(self, node: ast.AST):
-        """AST 전체를 순회하여 함수 정의(ast.FunctionDef)만 미리 수집."""
+        """Traverses the entire AST and pre-collects only function definitions (ast.FunctionDef)."""
         for child in ast.walk(node):
             if isinstance(child, ast.FunctionDef):
                 self.function_definitions[child.name] = child
 
     def _is_major_statement(self, stmt: ast.stmt):
         """
-        '주요 구문' 여부 판별:
+        Determines whether the statement is a 'major statement':
         - if, for, while, function def, return
-        - (중첩을 포함하여) 사용자 정의 함수 호출이 있는지 검사
+        - Checks for user-defined function calls (including nested ones).
         """
-        # 1) if / for / while / function def / return 등은 바로 True
+        # 1) if / for / while / function def / return are immediately True
         if isinstance(stmt, (ast.If, ast.For, ast.While, ast.FunctionDef, ast.Return)):
             return True
 
-        # 2) stmt 내부(자식 노드 전부)를 순회하여 ast.Call 탐색
+        # 2) Traverse all child nodes of stmt to find ast.Call
         for node in ast.walk(stmt):
             if isinstance(node, ast.Call):
-                # 어떤 함수가 호출됐는지 이름을 추출
+                # Extract the name of the function being called
                 try:
                     func_name = ast.unparse(node.func)
                 except Exception:
                     func_name = None
 
-                # 사용자 정의 함수 목록에 있는 이름인지 판별
+                # Check if the name is in the list of user-defined functions
                 if func_name and func_name in self.function_definitions:
                     return True
 
-        # 3) 위 조건에 걸리지 않으면 False
+        # 3) If it doesn't meet any of the above conditions, return False
         return False
 
-
     def _grouped_statements(self, stmts):
         """
-        구문 리스트를 순회하며 주요 구문을 단독 처리,
-        그 외 연속된 minor 구문들을 하나의 그룹(list)으로 묶어서 반환.
-        
-        예:
+        Iterates through a list of statements, processes major statements individually,
+        and groups consecutive minor statements into one group (list).
+
+        Example:
         [Expr(1), Expr(2), If(...), Expr(3), Expr(4), While(...)]
         -> [[Expr(1), Expr(2)], If(...), [Expr(3), Expr(4)], While(...)]
         """
@@ -124,8 +123,8 @@ class CodeFlowchartGenerator:
 
     def _process_body(self, stmts, cluster, start_node, return_points=None):
         """
-        여러 statement(또는 statement 그룹)을 순서대로 처리해
-        마지막 노드를 반환한다.
+        Processes multiple statements (or statement groups) sequentially
+        and returns the last node.
         """
         if return_points is None:
             return_points = []
@@ -140,23 +139,22 @@ class CodeFlowchartGenerator:
     
     def _process_function_call(self, call_node, current_node, cluster, edge_label=""):
         """
-        call_node: ast.Call
-        현재 코드에서의 함수 호출을 처리하고,
-        만약 인자들 중에 또 다른 ast.Call(=중첩 함수 호출)이 있다면 재귀적으로 처리.
+        Processes a function call at the current code position.
+        If there are nested function calls among the arguments, processes them recursively.
         """
         func_name = ast.unparse(call_node.func)
         expr_content = ast.unparse(call_node)
 
-        # 1) 사용자 정의 함수?
+        # 1) User-defined function?
         if func_name in self.function_definitions:
-            # Call 노드 생성
+            # Create a Call node
             expr_node = self._add_node(f"Call: {expr_content}", "box", cluster=cluster, prefix="call")
             self._add_edge(current_node, expr_node, cluster=cluster, label=edge_label)
 
-            # 함수 클러스터에 함수 노드들 만들어주기
+            # Generate nodes for the function's internal flow in the function cluster
             func_def = self.function_definitions[func_name]
             if func_name not in self.function_flow_nodes:
-                # 아직 해당 함수에 대한 노드가 없다면 새로 생성
+                # If no nodes exist for the function, create them
                 func_start_node = self._add_node(
                     f"Function: {func_name}", "ellipse",
                     cluster=self.function_cluster, prefix="function"
@@ -171,33 +169,33 @@ class CodeFlowchartGenerator:
             else:
                 func_start_node = self.function_flow_nodes[func_name][0]
 
-            # 메인 클러스터에서 함수 호출 노드 -> 함수 시작 노드(ellipse) 점선 연결
+            # Connect the function call node to the function start node in the main cluster with a dashed line
             self._add_edge(expr_node, func_start_node, label="Call", cluster=self.main_cluster, style="dashed")
 
-            # 함수 내부 return 지점들 -> 호출 노드로 복귀
+            # Link function return points back to the function call node
             for return_node in self.function_flow_nodes[func_name][2]:
                 self._add_edge(return_node, expr_node, label="Return", style="dashed")
 
-            # 2) 인자들 중에 또 다른 함수 호출이 있는지 확인 (중첩)
+            # 2) Check for nested function calls among arguments
             for arg in call_node.args:
                 if isinstance(arg, ast.Call):
-                    # 재귀 호출
+                    # Process recursively
                     self._process_function_call(arg, expr_node, cluster, edge_label="Arg Call")
 
             return expr_node
 
         else:
-            # 외부 함수 or 라이브러리 함수
+            # External or library function
             expr_node = self._add_node(f"Expression: {expr_content}", "box", cluster=cluster, prefix="expr")
             self._add_edge(current_node, expr_node, cluster=cluster, label=edge_label)
 
-            # 인자들 중 중첩 함수 호출 탐색
+            # Check for nested function calls in arguments
             for arg in call_node.args:
                 if isinstance(arg, ast.Call):
-                    # 재귀로 처리
+                    # Process recursively
                     self._process_function_call(arg, expr_node, cluster, edge_label="Arg Call")
 
-            # keyword args 처리
+            # Process keyword arguments
             for kw in call_node.keywords:
                 if isinstance(kw.value, ast.Call):
                     self._process_function_call(kw.value, expr_node, cluster, edge_label="Arg Call")
@@ -206,35 +204,35 @@ class CodeFlowchartGenerator:
 
     def _process_node(self, node, current_node, cluster=None, return_points=None, edge_label=""):
         """
-        AST 노드(또는 노드 리스트)를 받아서
-        대응되는 Flow 차트 노드를 생성, 연결 후 다음 노드를 반환.
+        Processes an AST node (or list of nodes) and generates corresponding
+        flowchart nodes, connects them, and returns the next node.
         """
         if return_points is None:
             return_points = []
 
-        # 1) 여러 minor statements가 묶인 list
+        # 1) List of multiple minor statements
         if isinstance(node, list):
             expr_node = self._add_node("Expression", "box", cluster=cluster, prefix="expr")
             self._add_edge(current_node, expr_node, cluster=cluster, label=edge_label)
             return expr_node
 
-        # 2) FunctionDef(함수 정의)
+        # 2) FunctionDef (function definition)
         elif isinstance(node, ast.FunctionDef):
-            # 함수 정의 노드는 메인 코드 흐름 상에서 그냥 지나침.
+            # Function definition nodes are bypassed in the main code flow
             return current_node
 
-        # 3) Expression(함수 호출/단순 표현식 등)
+        # 3) Expression (function call/simple expressions, etc.)
         elif isinstance(node, ast.Expr):
             if isinstance(node.value, ast.Call):
                 return self._process_function_call(node.value, current_node, cluster)
             else:
-                # 단순 표현식
+                # Simple expression
                 expr_content = ast.unparse(node.value)
                 expr_node = self._add_node(f"Expression: {expr_content}", "box", cluster=cluster, prefix="expr")
                 self._add_edge(current_node, expr_node, cluster=cluster, label=edge_label)
                 return expr_node
 
-        # 4) If
+        # 4) If statement
         elif isinstance(node, ast.If):
             condition = ast.unparse(node.test)
             if_node = self._add_node(f"If: {condition}", "diamond", cluster=cluster, prefix="if")
@@ -244,7 +242,7 @@ class CodeFlowchartGenerator:
             if_end_node = body_node
             is_first_true_edge = True
 
-            # if 본문
+            # If body
             grouped_body = self._grouped_statements(node.body)
             for g in grouped_body:
                 edge_label_for_body = "True" if is_first_true_edge else ""
@@ -254,15 +252,15 @@ class CodeFlowchartGenerator:
 
             end_nodes = [if_end_node]
 
-            # else / elif 처리
+            # Else/Elif handling
             if node.orelse:
                 first_else_node = node.orelse[0] if node.orelse else None
                 if isinstance(first_else_node, ast.If):
-                    # Elif인 경우
+                    # If it's an elif
                     elif_end_node = self._process_node(
                         first_else_node, if_node, cluster=cluster, return_points=return_points, edge_label="False"
                     )
-                    # 첫 elif의 나머지 else 구문은 first_else_node.orelse로 처리됨
+                    # Remaining else of the first elif is processed as first_else_node.orelse
                     node.orelse = first_else_node.orelse
                     end_nodes.append(elif_end_node)
                 else:
@@ -274,48 +272,48 @@ class CodeFlowchartGenerator:
                         else_end_node = self._process_node(g, else_end_node, cluster=cluster, return_points=return_points)
                     end_nodes.append(else_end_node)
 
-            # If 종료(merge) 지점
+            # Merge point at the end of If
             merge_node = self._add_node("If End", "circle", cluster=cluster, prefix="if_end")
             for end_node in end_nodes:
-                if self._get_node_prefix(end_node) != "return":  # return 노드는 merge로 연결 안 함
+                if self._get_node_prefix(end_node) != "return":  # Do not connect return nodes to merge
                     self._add_edge(end_node, merge_node, cluster=cluster)
 
             return merge_node
 
-        # 5) For
+        # 5) For loop
         elif isinstance(node, ast.For):
             target = ast.unparse(node.target)
             iter_ = ast.unparse(node.iter)
-            for_node = self._add_node(f"For: {target} in {iter_}", "box", cluster=cluster, prefix="for")
+            for_node = self._add_node(f"For: {target} in {iter_}", "parallelogram", cluster=cluster, prefix="for")
             self._add_edge(current_node, for_node, cluster=cluster)
 
-            # for 루프 본문
+            # For loop body
             body_node = for_node
             body_node = self._process_body(node.body, cluster, body_node, return_points)
 
-            # 루프 반복(마지막에서 다시 for_node로)
+            # Loop back to the start of the for node
             self._add_edge(body_node, for_node, "Repeat", cluster=cluster, style="dotted")
             return for_node
 
-        # 6) While
+        # 6) While loop
         elif isinstance(node, ast.While):
             condition = ast.unparse(node.test)
-            while_node = self._add_node(f"While: {condition}", "diamond", cluster=cluster, prefix="while")
+            while_node = self._add_node(f"While: {condition}", "parallelogram", cluster=cluster, prefix="while")
             self._add_edge(current_node, while_node, cluster=cluster)
 
-            # while 본문
+            # While loop body
             body_node = while_node
             body_node = self._process_body(node.body, cluster, body_node, return_points)
 
-            # 반복
+            # Loop back to the start of the while node
             self._add_edge(body_node, while_node, "Repeat", cluster=cluster, style="dotted")
 
-            # False일 때 탈출
+            # Exit point when condition is False
             exit_node = self._add_node("While End", "circle", cluster=cluster, prefix="while_end")
             self._add_edge(while_node, exit_node, "False", cluster=cluster)
             return exit_node
 
-        # 7) Return
+        # 7) Return statement
         elif isinstance(node, ast.Return):
             return_value = ast.unparse(node.value) if node.value else "None"
             return_node = self._add_node(f"Return: {return_value}", "box", cluster=cluster, prefix="return")
@@ -323,45 +321,46 @@ class CodeFlowchartGenerator:
             return_points.append(return_node)
             return return_node
 
-        # 8) 기타
+        # 8) Other cases
         else:
             return current_node
 
     def generate_flowchart(self, output_file="code_flowchart"):
-        # Start 노드
+        # Start node
         start_node = self._add_node("Start", "ellipse", cluster=self.main_cluster, prefix="start", rank="source")
         current_node = start_node
 
-        # 최상위 레벨 구문을 그룹화 처리
+        # Group and process top-level statements
         grouped_top_level = self._grouped_statements(self.tree.body)
         for g in grouped_top_level:
             current_node = self._process_node(g, current_node, cluster=self.main_cluster)
 
-        # End 노드
+        # End node
         end_node = self._add_node("End", "ellipse", cluster=self.main_cluster, prefix="end", rank="sink")
         self._add_edge(current_node, end_node, cluster=self.main_cluster)
 
-        # 서브그래프 추가
+        # Add subgraphs
         self.graph.subgraph(self.function_cluster)
         self.graph.subgraph(self.main_cluster)
 
-        # DOT 파일 출력
+        # Output DOT file
         dot_file = f"{output_file}.dot"
         with open(dot_file, "w", encoding="utf-8") as file:
             file.write(self.graph.source)
         print(f"DOT file saved as {dot_file}")
 
-        # PNG 이미지 렌더
+        # Render PNG image
         self.graph.render(filename=output_file, format='png', cleanup=True)
         print(f"Flowchart saved as {output_file}.png")
 
-if __name__=="__main__":
+if __name__ == "__main__":
     sample_code = """
 def isOdd(num):
-    if num%2==0:
-        return "It's not odd!!!"
+    if not num%2==0:
+        return True
     else:
-        return "It's odd!!!"
+        print("It's not odd!!!")
+    return False
 
 for i in range(3):
     print(isOdd(i))
